/*******************************************************************************
 * Copyright (C) 2007 The University of Manchester   
 * 
 *  Modifications to the initial code base are copyright of their
 *  respective authors, or their employers as appropriate.
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *    
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *    
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 ******************************************************************************/
package net.sf.taverna.t2.activities.wsdl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.taverna.t2.activities.testutils.ActivityInvoker;

import org.junit.Test;
public class WSDLActivityTest {
	
	
	
	public WSDLActivity initActivity(String wsdl, String operation) throws Exception {
		WSDLActivity activity=new WSDLActivity();
		WSDLActivityConfigurationBean bean = new WSDLActivityConfigurationBean();
		bean.setWsdl(WSDLTestConstants.WSDL_TEST_BASE+wsdl);
		bean.setOperation(operation);
		activity.configure(bean);
		return activity;
	}
	
	@Test
	public void testBasicInvocation() throws Exception {
		WSDLActivity activity = initActivity("menagerie-complex-rpc.wsdl", "createPerson");
		Map<String,Object> inputMap = new HashMap<String,Object>();
		Map<String, Class<?>> outputMap = new HashMap<String, Class<?>>();
		outputMap.put("out", String.class);
		Map<String,Object> results = ActivityInvoker.invokeAsyncActivity(activity, inputMap, outputMap);
		assertEquals(1,results.size());
		assertNotNull(results.get("out"));
		assertTrue(results.get("out") instanceof String);
	}
	
	@Test
	public void testWithArrayReturned() throws Exception {
		WSDLActivity activity = initActivity("KEGG.wsdl", "get_pathways_by_genes");
		Map<String,Object> inputMap = new HashMap<String, Object>();
		List<String> inputs = new ArrayList<String>();
		inputs.add("eco:b0077");
		inputs.add("eco:b0078");
		inputMap.put("genes_id_list", inputs);
		Map<String, Class<?>> outputMap = new HashMap<String, Class<?>>();
		outputMap.put("return", String.class);
		Map<String,Object> results = ActivityInvoker.invokeAsyncActivity(activity, inputMap, outputMap);
		assertEquals(1,results.size());
		assertNotNull(results.get("return"));
		assertTrue(results.get("return") instanceof List);
	}
	
}
