/*******************************************************************************
 * Copyright (C) 2007 The University of Manchester   
 * 
 *  Modifications to the initial code base are copyright of their
 *  respective authors, or their employers as appropriate.
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *    
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *    
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 ******************************************************************************/
package net.sf.taverna.t2.partition;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Icon;

/**
 * Abstract class that defines the Action handler responsible for adding new ActivityQueries.
 * @author Stuart Owen
 *
 */
public abstract class AddQueryActionHandler extends AbstractAction {
	
	private SetModelChangeListener<ActivityItem> setModelChangeListener;
	
	public AddQueryActionHandler() {
		super();
		putValue(AbstractAction.NAME,getText());
		putValue(AbstractAction.SMALL_ICON,getIcon());
	}

	public AddQueryActionHandler(String name, Icon icon) {
		super(name, icon);
	}

	public AddQueryActionHandler(String name) {
		super(name);
	}
	
	public void setSetModelChangeListener(SetModelChangeListener<ActivityItem> listener) {
		this.setModelChangeListener=listener;
	}
	
	protected SetModelChangeListener<ActivityItem> getSetModelChangeListener() {
		return this.setModelChangeListener;
	}
	
	@SuppressWarnings("unchecked")
	protected void addQuery(final Query<?> query) {
		query.addSetModelChangeListener((SetModelChangeListener)getSetModelChangeListener());
		Thread t = new Thread("Add Query thread") {
			@Override
			public void run() {
				query.doQuery();
			}
		};
		
		t.start();
	}

	/**
	 * The implementation should invoke the appropriate action to handle adding a new query
	 */
	public abstract void actionPerformed(ActionEvent actionEvent);
	
	protected abstract String getText();
	protected abstract Icon getIcon();

}
